package com.androidmc;

import java.io.File;
import java.util.ArrayList;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class AndroidMC extends Activity {
	private ListView file_expl;
	private ImageButton btn_back;
	private TextView info;
	private File parentDir;
	private int list_pos;

	private ArrayList<String> groups = new ArrayList<String>();
    private ArrayList<String> dir_list = new ArrayList<String>();
    private ArrayList<String> pic_list = new ArrayList<String>();
    private ArrayList<String> audio_plst = new ArrayList<String>();
	private ArrayList<String> video_plst = new ArrayList<String>();
    
    public static final int NONE = 0;
    public static final int AUDIO = 1;
    public static final int VIDEO = 2;
    public static final int PICTURES = 3;
    public static final int DIR = 4;
    
    private static final int SHOW_DIALOG = 5;
    private static final int DELETE_DIALOG = 7;
    
    private int slctGroup; 
    
    private MCSInterface mpInterface;
    
    @Override
	protected Dialog onCreateDialog(int id) {
		switch(id) {
		case SHOW_DIALOG:
			return new AlertDialog.Builder(AndroidMC.this)
            .setTitle(R.string.select_dialog)
            .setItems(R.array.slct_dialog_items, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int which) {
                	
                	if(which == 0) {
                		switch(slctGroup)
            			{
            			case AUDIO:
            				Intent intent = new Intent(AndroidMC.this, AudioPlayer.class);
            				intent = intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
            				//intent = intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            				AndroidMC.this.startActivity(intent);
            				break;
            			case VIDEO:
            				Intent i = new Intent(AndroidMC.this, VideoPlayer.class);
            				i.putStringArrayListExtra("PLAYLIST", video_plst);
            				i.putExtra("POS", list_pos);
            				i.putExtra("DIR", parentDir.getAbsolutePath());
            				try {
            					if(mpInterface.MPStatus() == MCService.PLAYING || mpInterface.MPStatus() == MCService.PAUSED)
            						mpInterface.stop();
            				}
            				catch(RemoteException e) {}
            				startActivity(i);
            				break;
            			case PICTURES:
            				Intent picIntent = new Intent(AndroidMC.this, ImageViewer.class);
            				picIntent.putStringArrayListExtra("IMG", pic_list);
            				picIntent.putExtra("DIR", parentDir.getAbsolutePath() + "/");
            				picIntent.putExtra("POS", list_pos);
            				startActivity(picIntent);
            				break;
            			default:
            				new AlertDialog.Builder(AndroidMC.this).setMessage("Can't open file!").show();
            				break;
            			}
                	}
                	else if(which == 1) {
                		showDialog(DELETE_DIALOG);
                	}
                }
            }).create();
		case DELETE_DIALOG:
			return new AlertDialog.Builder(AndroidMC.this)
            .setIcon(R.drawable.alert_dialog_icon)
            .setTitle(R.string.alert_dialog_title)
            .setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {

                    switch(slctGroup) {
                    case AUDIO:
                    	DeleteFile(new File(parentDir.getAbsolutePath() + "/" + audio_plst.get(list_pos)));
                    	audio_plst.remove(list_pos);
                    	try {
                    		mpInterface.delSong(list_pos);
                    	}
                    	catch(RemoteException e) {}
                    	showAudioPlst();
                    	break;
                    case VIDEO:
                    	DeleteFile(new File(parentDir.getAbsolutePath() + "/" + video_plst.get(list_pos)));
                    	video_plst.remove(list_pos);
                    	showVideoPlst();
                    	break;
                    case PICTURES:
                    	DeleteFile(new File(parentDir.getAbsolutePath() + "/" + pic_list.get(list_pos)));
                    	pic_list.remove(list_pos);
                    	showPicPlst();
                    	break;
                    }
                }
            })
            .setNegativeButton(R.string.alert_dialog_cancel, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                }
            }).create();
		}
    	return null;
	}

	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        file_expl = (ListView)findViewById(R.id.file_list);
        file_expl.setOnItemClickListener(mPlstItemClick);
        btn_back = (ImageButton)findViewById(R.id.btn_back);
        btn_back.setOnClickListener(mBack);
        info = (TextView)findViewById(R.id.file_info);
        
        startService(new Intent("com.androidmc.MC_SERVICE"));
        
        this.bindService(new Intent(MCSInterface.class.getName()), mConnection, Context.BIND_AUTO_CREATE);
    }
    
	private ServiceConnection mConnection = new ServiceConnection() {
		
		public void onServiceConnected(ComponentName className, IBinder service) {
			mpInterface = MCSInterface.Stub.asInterface((IBinder)service);
			browseTo(new File("/sdcard"));
		}
		
		public void onServiceDisconnected(ComponentName className) {
			mpInterface = null;
		}
	};
	
	private void back()
    {
    	switch(slctGroup)
    	{
    	case NONE:
    	case AUDIO:
    	case VIDEO:
    	case PICTURES:
    	case DIR:
    		if(parentDir.getParent() != null && parentDir.getAbsolutePath() != "/")
    			browseTo(parentDir);
    		break;
    	}
    }
	
    private void browseTo(File dir)
    {
    	if (dir.isDirectory()){
       	  	parentDir = dir;
       	  	try {
       	  		mpInterface.setDirPath(parentDir.getAbsolutePath() + "/");
       	  	}
       	  	catch(RemoteException e) {
       	  		Log.e(getString(R.string.app_name), e.getMessage());
       	  	}
            setTitle(dir.getAbsolutePath());
            fill(dir.listFiles());
        }
    }

    private void fill(File[] files) 
    {
         dir_list.clear();
         pic_list.clear();
         groups.clear();
         audio_plst.clear();
         video_plst.clear();
         
         if(parentDir.getParent() != null && parentDir.getAbsolutePath() != "/sdcard")
        	 btn_back.setClickable(true);
         else
        	 btn_back.setClickable(false);
         
         String f = "";
         for (File file : files) {
        	 f = file.getName();
        	 
        	 try {
	        	 if (MCUtils.isAudioFile(f)) {
	        		 mpInterface.addAudioPlaylist(f);
	        		 audio_plst.add(f);
	        	 }
	        	 else if (MCUtils.isVideoFile(f)) {
	        		 video_plst.add(f);
	        	 }
	        	 else if (MCUtils.isPictureFile(f))
	        		 pic_list.add(f);
	        	 else if(file.isDirectory())
	        		 dir_list.add(f);
        	 }
        	 catch(RemoteException e) {
        		Log.e(getString(R.string.app_name), e.getMessage());
        	 }
         }
         
         if(audio_plst.size() > 0)
        	 groups.add("AUDIO");
         if(video_plst.size() > 0)
        	 groups.add("VIDEO");
         if(pic_list.size() > 0)
        	 groups.add("PICTURES");
         if(dir_list.size() > 0)
        	 groups.add("DIR");
         
         slctGroup = NONE;
         
         ArrayAdapter<String> directoryList = new ArrayAdapter<String>(this,android.R.layout.simple_list_item_1, groups);
         file_expl.setAdapter(directoryList);
    }
    
    private void DeleteFile(File f) {
    	boolean deleted = f.delete();
    	if(!deleted)
    		new AlertDialog.Builder(AndroidMC.this).setMessage("Read only. Can't delete file!").show();
    }
    
    private void showAudioPlst()
    {
    	slctGroup = AUDIO;
    	ArrayAdapter<String> audioAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, audio_plst);
    	file_expl.setAdapter(audioAdapter);
    }
    
    private void showVideoPlst()
    {
    	slctGroup = VIDEO;
    	ArrayAdapter<String> videoAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, video_plst);
    	file_expl.setAdapter(videoAdapter);
    }
    
    private void showPicPlst()
    {
    	slctGroup = PICTURES;
    	//ArrayAdapter<String> mPhoto = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, pic_list);
    	PictureAdapter mPhoto = new PictureAdapter(this, pic_list, parentDir.getAbsolutePath() + "/");
    	file_expl.setAdapter(mPhoto);
    }
    
    private void showDir()
    {
    	slctGroup = DIR;
    	ArrayAdapter<String> dirAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, dir_list);
    	file_expl.setAdapter(dirAdapter);
    }
 
    private OnItemClickListener mPlstItemClick = new OnItemClickListener()
    {
		@Override
		public void onItemClick(AdapterView l, View v, int position, long id) {
			switch(slctGroup)
			{
			case NONE:
				String groupName = groups.get(position);
				if (groupName == "AUDIO")
					showAudioPlst();
				else if (groupName == "VIDEO")
					showVideoPlst();
				else if (groupName == "PICTURES")
					showPicPlst();
				else if(groupName == "DIR")
					showDir();
				break;
			case AUDIO:
				try {
					mpInterface.setCurrentPosition(position);
				}
				catch (RemoteException e) {
					Log.e(getString(R.string.app_name), e.getMessage());
				}
			case VIDEO:
			case PICTURES:
				list_pos = position;
				showDialog(SHOW_DIALOG);
				break;
			case DIR:
				browseTo(new File(parentDir.getAbsolutePath() + "/" + dir_list.get(position)));
				break;
			}
		}
    };
    
    private OnClickListener mBack = new OnClickListener()
    {
    	public void onClick(View v)
    	{
    		back();
    	}
    };
}