package com.androidmc;

import java.io.ByteArrayInputStream;
import java.io.File;

import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.TagField;
import org.jaudiotagger.tag.TagFieldKey;
import org.jaudiotagger.tag.datatype.DataTypes;
import org.jaudiotagger.tag.id3.AbstractID3v2Frame;
import org.jaudiotagger.tag.id3.ID3v1Tag;
import org.jaudiotagger.tag.id3.framebody.FrameBodyAPIC;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

public class AudioPlayer extends Activity implements SeekBar.OnSeekBarChangeListener {
	private ImageButton btnPlay;
	private ImageButton btnStop;
	private ImageButton btnPrev;
	private ImageButton btnNext;
	private ImageView albumCover;
	private TextView txtArtist, txtAlbum, txtSong;
	private TextView song_duration, song_curTimePos;
	private SeekBar av_seekBar;
	
	private Thread tProgress = null;
	private Handler mHandler = new Handler();
	private int mProgressStatus = 0;
	int songDuration = 0;
	
	private MCSInterface mpInterface;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.ap_layout);
		
		btnPlay = (ImageButton)findViewById(R.id.av_btn_play);
		btnStop = (ImageButton)findViewById(R.id.av_btn_stop);
		btnPrev = (ImageButton)findViewById(R.id.av_btn_prev);
		btnNext = (ImageButton)findViewById(R.id.av_btn_next);
		btnPlay.setOnClickListener(mPlay);
		btnStop.setOnClickListener(mStop);
		btnPrev.setOnClickListener(mPrev);
		btnNext.setOnClickListener(mNext);
		
		albumCover = (ImageView)findViewById(R.id.av_album_cover);
		txtArtist = (TextView)findViewById(R.id.av_tag_artist);
		txtAlbum = (TextView)findViewById(R.id.av_tag_album);
		txtSong = (TextView)findViewById(R.id.av_tag_song);
		song_duration = (TextView)findViewById(R.id.av_duration);
		song_curTimePos = (TextView)findViewById(R.id.av_cur_timepos);
		av_seekBar = (SeekBar)findViewById(R.id.av_seekbar);
		av_seekBar.setOnSeekBarChangeListener(this);
		
		this.bindService(new Intent(MCSInterface.class.getName()), mConnection, Context.BIND_AUTO_CREATE);
	}
	
	@Override
	protected void onStop() {
		tProgress.interrupt();
		unbindService(mConnection);
		super.onStop();
	}

	@Override
	protected void onPause() {
		super.onPause();
		tProgress.suspend();
	}

	@Override
	protected void onRestart() {
		super.onRestart();
		this.bindService(new Intent(MCSInterface.class.getName()), mConnection, Context.BIND_AUTO_CREATE);
	}

	@Override
	protected void onResume() {
		super.onResume();
		if(tProgress != null)
			tProgress.resume();
	}

	private ServiceConnection mConnection = new ServiceConnection() {
		
		public void onServiceConnected(ComponentName className, IBinder service) {
			mpInterface = MCSInterface.Stub.asInterface((IBinder)service);
			SetID3TagStuff();
			try {
				mpInterface.playAudioFile();
				songDuration = mpInterface.getSongDuration();
			}
			catch (RemoteException e) {}
			av_seekBar.setMax(songDuration);
			song_duration.setText("/ " + MCUtils.MillisecToMinSec(songDuration));
			tProgress = new Thread(new Runnable() {
            	public void run() {
	        		while (mProgressStatus < songDuration) {
	        			mProgressStatus = GetProgress();
	        			mHandler.post(new Runnable() {
	        				public void run() {
			        			av_seekBar.setProgress(mProgressStatus);
			        			song_curTimePos.setText(MCUtils.MillisecToMinSec(mProgressStatus));
			        		}
	        			});
	        			try {
	        				Thread.sleep(1000);
	        			}
	        			catch(InterruptedException e) {
	        				return;
	        			}
	        		}
            	}
            });
            tProgress.setPriority(Thread.MIN_PRIORITY);
            tProgress.start();
            btnPlay.setImageResource(R.drawable.pause);
		}
		
		public void onServiceDisconnected(ComponentName className) {
			mpInterface = null;
		}
	};
	
	private int GetProgress() {
		int retValue = 0;
		try {
			retValue = mpInterface.getCurrentPosition();
		}
		catch (RemoteException e) {}
		return retValue;
	}
	
	private void SetID3TagStuff() {
		try {
			File song = new File(mpInterface.getCurSongPath());
			MP3File f = (MP3File)AudioFileIO.read(song);
			ID3v1Tag tag = f.getID3v1Tag();
			String song_artist = tag.getFirstArtist();
			String song_album = tag.getFirstAlbum();
			String song_title = tag.getFirstTitle();
			
			if (song_artist.length() == 0)
				song_artist = "Unknown artist";
			if (song_album.length() == 0)
				song_album = "Unknown album";
			if (song_title.length() == 0)
				song_title = song.getName();
			
			txtArtist.setText(song_artist);
			txtAlbum.setText(song_album);
			txtSong.setText(song_title);
			
			mpInterface.setSongArtist(song_artist);
			mpInterface.setSongAlbum(song_album);
			mpInterface.setSongTitle(song_title);
			
			Bitmap bi = null;
			TagField imageField = f.getTag().get(TagFieldKey.COVER_ART).get(0);
			if(imageField instanceof AbstractID3v2Frame) {
				FrameBodyAPIC imageFrameBody = (FrameBodyAPIC)((AbstractID3v2Frame)imageField).getBody();
			    if(!imageFrameBody.isImageUrl()) {
			        byte[] imageRawData = (byte[])imageFrameBody.getObjectValue(DataTypes.OBJ_PICTURE_DATA);
			        bi=BitmapFactory.decodeStream(new ByteArrayInputStream(imageRawData));
			        albumCover.setImageBitmap(bi);
			    }
			}
		}
		catch (Exception e) {
			albumCover.setImageResource(R.drawable.album_cover);
		}
		
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		
		MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.ap_menu, menu);
        
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId()) {
		case R.id.ap_library:
			Intent intent = new Intent(AudioPlayer.this, AndroidMC.class);
			intent = intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			intent = intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
			startActivity(intent);
			return true;
		}
		return false;
	}

	private OnClickListener mPlay = new OnClickListener()
    {
    	public void onClick(View v)
    	{
    		try {
    			int mpStatus = mpInterface.MPStatus();
	    		if (mpStatus == MCService.STOPPED) {
	    			mpInterface.playAudioFile();
	    			btnPlay.setImageResource(R.drawable.pause);
	    			tProgress.resume();
	    		}
	    		else if (mpStatus == MCService.PLAYING) {
	    			mpInterface.pause();
	    			btnPlay.setImageResource(R.drawable.play);
	    			tProgress.suspend();
	    		}
	    		else if (mpStatus == MCService.PAUSED) {
	    			mpInterface.resume();
	    			btnPlay.setImageResource(R.drawable.pause);
	    			tProgress.resume();
	    		}
    		}
    		catch(RemoteException e) {
        		Log.e(getString(R.string.app_name), e.getMessage());
        	}
    	}
    };
    
    private OnClickListener mStop = new OnClickListener()
    {
    	public void onClick(View v)
    	{
    		try {
    			mpInterface.stop();
    			btnPlay.setImageResource(R.drawable.play);
    			tProgress.suspend();
    		}
    		catch(RemoteException e) {
        		Log.e(getString(R.string.app_name), e.getMessage());
        	}
    	}
    };
    
    private OnClickListener mPrev = new OnClickListener()
    {
    	public void onClick(View v)
    	{
    		try {
    			mpInterface.skipBack();
    			int newDuration = mpInterface.getSongDuration();
    			if(newDuration == songDuration)
    				return;
    			else {
    				songDuration = newDuration;
    				SetID3TagStuff();
    			}
    		}
    		catch(RemoteException e) {
        		Log.e(getString(R.string.app_name), e.getMessage());
        	}
    	}
    };
    
    private OnClickListener mNext = new OnClickListener()
    {
    	public void onClick(View v)
    	{
    		try {
    			mpInterface.skipForward();
    			songDuration = mpInterface.getSongDuration();
    			SetID3TagStuff();
    		}
    		catch(RemoteException e) {
        		Log.e(getString(R.string.app_name), e.getMessage());
        	}
    	}
    };  
    
	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch) {
    	//if(tProgress != null)
    		//tProgress.resume();
    }

	public void onStartTrackingTouch(SeekBar seekBar) {
    	if(tProgress.getState() != Thread.State.WAITING)
    		tProgress.suspend();
    }

	public void onStopTrackingTouch(SeekBar seekBar) {
		try {
    		mpInterface.seekTo(seekBar.getProgress());
    	}
    	catch (RemoteException e) {}
    	tProgress.resume();
    }
}
