package com.androidmc;

import java.util.ArrayList;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.os.Bundle;
import android.view.Display;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ViewFlipper;
import android.widget.ImageView.ScaleType;

public class ImageViewer extends Activity {
	private ArrayList<String> img_list;
	private String path;
	private int curPos = 0;
	
	private TransparentPanel controlPanel;
	private ViewFlipper flipper;
	private ImageButton btn_next, btn_prev, btn_rotate;
	private Button btn_zoom_in, btn_zoom_out;
	private ImageView image;
	
	private float zoom = 0.0f;
	private int rotDeg = 0;
	private float scaleWidth = 0.0f, scaleHeight = 0.0f;
	private float imageWidth = 0.0f, imageHeight = 0.0f;
	private float newHeight = 0.0f, newWidth = 0.0f;
	private int screenWidth, screenHeight;
	private int scrollX = 0, scrollY = 0;
	private float mx,my; 
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.imageviewer_layout);
		
		Bundle extr = getIntent().getExtras();
		path = extr.getString("DIR");
		img_list = extr.getStringArrayList("IMG");
		curPos = extr.getInt("POS");
		
		image = (ImageView)findViewById(R.id.image);
		controlPanel = (TransparentPanel)findViewById(R.id.transparent_panel);
		flipper = (ViewFlipper)findViewById(R.id.iv_flipper);
		btn_next = (ImageButton)findViewById(R.id.iv_next);
		btn_prev = (ImageButton)findViewById(R.id.iv_prev);
		btn_rotate = (ImageButton)findViewById(R.id.iv_rotate);
		btn_zoom_in = (Button)findViewById(R.id.iv_zoomin);
		btn_zoom_out = (Button)findViewById(R.id.iv_zoomout);
		
		flipper.setInAnimation(this, android.R.anim.fade_in);
		flipper.setOutAnimation(this, android.R.anim.fade_out);
		flipper.setOnTouchListener(mTouchCntrlPanel);
		image.setOnTouchListener(mTouchScroll);
		btn_next.setOnClickListener(mNextImg);
		btn_prev.setOnClickListener(mPrevImg);
		btn_rotate.setOnClickListener(mRotateImg);
		btn_zoom_in.setOnClickListener(mZoomInImg);
		btn_zoom_out.setOnClickListener(mZoomOutImg);
		
		Display disp = getWindow().getWindowManager().getDefaultDisplay();
		screenWidth = disp.getWidth();
		screenHeight = disp.getHeight() - 50;
		
		ShowImg();
	}
	
	private void ShowImg() {
		BitmapFactory.Options o = new BitmapFactory.Options();
		o.inSampleSize = 2;
		Bitmap cur_bm = BitmapFactory.decodeFile(path + img_list.get(curPos), o);
		imageWidth = (float)cur_bm.getWidth();
		imageHeight = (float)cur_bm.getHeight();
		newHeight = imageHeight / (imageWidth / screenWidth);
		newWidth = screenWidth;
		scaleWidth = screenWidth / imageWidth;
		scaleHeight = newHeight / imageHeight;
		image.setImageBitmap(cur_bm);
		setTransformationMatrix(0.0f, (screenHeight - newHeight)/2);
		ScrollBack();
	}
	
	private void NextImg() {
		if(curPos+1 < img_list.size()) {
			curPos++;
			ShowImg();
		}
		return;
	}
	
	private void PrevImg() {
		if(curPos >= 1) {
			curPos--;
			ShowImg();
		}
		return;
	}
	
	private void setTransformationMatrix(float dx, float dy) {
		Matrix mtrx = new Matrix();
		mtrx.postScale(scaleWidth, scaleHeight);
		mtrx.postRotate((float)rotDeg);
		mtrx.postTranslate(dx, dy);
		image.setImageMatrix(mtrx);
		image.setScaleType(ScaleType.MATRIX);
		image.invalidate();
	}
	
	private void ScrollBack() {
		if(scrollX != 0) {
			image.scrollBy(-scrollX, 0);
			scrollX = 0;
		}
		if(scrollY != 0) {
			image.scrollBy(0, -scrollY);
			scrollY = 0;
		}
	}
	
	private void Zoom() {
		newHeight = scaleHeight * imageHeight;
		newWidth = scaleWidth * imageWidth;
		setTransformationMatrix((screenWidth - newWidth)/2, (screenHeight - newHeight)/2);
		ScrollBack();
	}
	
	private OnTouchListener mTouchCntrlPanel = new OnTouchListener() {

		@Override
		public boolean onTouch(View v, MotionEvent event) {
			if(event.getAction() == MotionEvent.ACTION_DOWN) {
				flipper.showNext();
				return true;
			}
			return false;
		}
		
	};
	
	private OnTouchListener mTouchScroll = new OnTouchListener() {

		@Override
		public boolean onTouch(View v, MotionEvent event) {
			float curX, curY;
			int objX, objY;
			switch(event.getAction()) {
			case MotionEvent.ACTION_DOWN:
				mx = event.getX();
				my = event.getY();
				return true;
			case MotionEvent.ACTION_MOVE:
				curX = event.getX();
				curY = event.getY();
				objX = (int)(mx - curX);
				objY = (int)(my - curY);
				int tx = Math.abs(scrollX + objX);
				int ty = Math.abs(scrollY + objY);
				int rx = (int)(newWidth - screenWidth)/2;
				int ry = (int)(newHeight - screenHeight)/2;
				if(rx <= 0 && ry <= 0)
					return true;
				else if(rx <= 0 && ty <= ry) {
					scrollY += objY;
					my = curY;
					image.scrollBy(0, objY);
				}
				else if(ry <= 0 && tx <= rx) {
					scrollX += objX;	
					mx = curX;
					image.scrollBy(objX, 0);
				}
				else if(tx <= rx && ty <= ry) {
					scrollX += objX;
					scrollY += objY;
					mx = curX;
					my = curY;
					image.scrollBy(objX, objY);
				}
				return true;
			case MotionEvent.ACTION_UP:
				curX = event.getX();
				curY = event.getY();
				objX = (int)(mx - curX);
				objY = (int)(my - curY);
				int ttx = Math.abs(scrollX + objX);
				int tty = Math.abs(scrollY + objY);
				int rrx = (int)(newWidth - screenWidth)/2;
				int rry = (int)(newHeight - screenHeight)/2;
				if(rrx <= 0 && rry <= 0)
					return true;
				else if(rrx <= 0 && tty <= rry) {
					scrollY += objY;
					image.scrollBy(0, objY);
				}
				else if(rry <= 0 && ttx <= rrx) {
					scrollX += objX;	
					image.scrollBy(objX, 0);
				}
				else if(ttx <= rrx && tty <= rry) {
					scrollX += objX;
					scrollY += objY;
					image.scrollBy(objX, objY);
				}
				return true;
			}
			return false;
		}
	};
	
	private OnClickListener mNextImg = new OnClickListener()
    {
    	public void onClick(View v) {
    		NextImg();
    	}
    };
	
	private OnClickListener mPrevImg = new OnClickListener()
    {
    	public void onClick(View v) {
    		PrevImg();
    	}
    };
    
    private OnClickListener mRotateImg = new OnClickListener()
    {
    	public void onClick(View v) {
    		ScrollBack();
    		if(rotDeg == 270)
    			rotDeg = 0;
    		else
    			rotDeg += 90;
    		switch(rotDeg) {
    		case 0:
    			setTransformationMatrix((screenWidth - newWidth)/2, (screenHeight - newHeight)/2);
    			break;
    		case 90:
    			setTransformationMatrix((screenWidth - newHeight)/2 + newHeight, (screenHeight - newWidth)/2);
    			break;	
    		case 180:
    			setTransformationMatrix((screenWidth - newWidth)/2 + newWidth, (screenHeight - newHeight)/2 + newHeight);
    			break;
    		case 270:
    			setTransformationMatrix((screenWidth - newHeight)/2 , (screenHeight - newWidth)/2 + newWidth);
    			break;
    		}
    		
    	}
    };
    
    private OnClickListener mZoomInImg = new OnClickListener()
    {
    	public void onClick(View v) {
    		zoom += 0.1f;
    		scaleWidth += zoom;
    		scaleHeight += zoom;
    		Zoom();
    	}
    };
    
    private OnClickListener mZoomOutImg = new OnClickListener()
    {
    	public void onClick(View v) {
    		if((zoom - 0.1f) >= 0) {
    			scaleWidth -= zoom;
    			scaleHeight -= zoom;
    			zoom -= 0.1f;
    			Zoom();
    		}
    	}
    };

	@Override
	public boolean onKeyUp(int keyCode, KeyEvent event) {
        switch (keyCode) {
        case KeyEvent.KEYCODE_DPAD_UP:
            image.scrollBy(0, -10);
            break;
        case KeyEvent.KEYCODE_DPAD_LEFT:
        	image.scrollBy(-10, 0);
            break;
        case KeyEvent.KEYCODE_DPAD_RIGHT:
        	image.scrollBy(10, 0);
            break;
        case KeyEvent.KEYCODE_DPAD_DOWN:
        	image.scrollBy(0, 10); 
            break;
        }
		return super.onKeyUp(keyCode, event);
	}
}
