package com.androidmc;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.id3.ID3v1Tag;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.os.IBinder;
import android.os.RemoteException;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;

public class MCService extends Service {
	private MediaPlayer mp = new MediaPlayer();
	private String dirPath;
	
	private ArrayList<String> audio_plst = new ArrayList<String>();
	private int curr_pos;
	
	private NotificationManager nm;
	private static final int NOTIFY_ID = R.string.remote_service;
	
	public static final int PLAYING = 0;
	public static final int PAUSED = 1;
	public static final int STOPPED = 3;
	private int mp_status;
	
	private String song_artist;
	private String song_album;
	private String song_title;
	private String curPlayingfile = "";
	private boolean hook = false;
		
	@Override
	public void onCreate() {
		super.onCreate();
		nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		mp_status = STOPPED;
		TelephonyManager tm = (TelephonyManager)getSystemService(TELEPHONY_SERVICE);
		tm.listen(mPhoneListen, PhoneStateListener.LISTEN_CALL_STATE);
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		mp.stop();
		mp.release();
		nm.cancel(NOTIFY_ID);
	}
	
	@Override
	public IBinder onBind(Intent arg0) {
		return mBinder;
	}

	private PhoneStateListener mPhoneListen = new PhoneStateListener() {

		@Override
		public void onCallStateChanged(int state, String incomingNumber) {
			switch(state) {
			case TelephonyManager.CALL_STATE_RINGING:
				if(mp_status == PLAYING)
					AP_pause();
				hook = true;
				break;
			case TelephonyManager.CALL_STATE_IDLE:
				if(hook) {
					hook = false;
					if(mp_status == PAUSED)
						AP_resume();
				}
				break;
			}
			super.onCallStateChanged(state, incomingNumber);
		}
	};
	
	private void playFile() {
		String file = audio_plst.get(curr_pos);
		if(mp_status == PLAYING || mp_status == PAUSED) {
			if(curPlayingfile == file)
				return;
		}
		try {
			curPlayingfile = file;
			mp.reset();
			mp.setDataSource(dirPath + file);
			mp.prepare();
			mp.start();
			mp.setOnCompletionListener(new OnCompletionListener() {
				public void onCompletion(MediaPlayer arg0) {
					nextFile();
				}
			});
			ShowNotification(R.drawable.playbackstart, song_title, song_artist);
			mp_status = PLAYING;
		}
		catch (IOException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}
	
	private void nextFile() {
		if(++curr_pos >= audio_plst.size()) {
			curr_pos = 0;
			nm.cancel(NOTIFY_ID);
		}
		else
			playFile();
	}
	
	private void prevFile() {
		if (mp.getCurrentPosition() < 3000 && curr_pos >= 1) {
			--curr_pos;
			playFile();
		} 
		else
			playFile();
	}
	
	private void AP_pause() {
		ShowNotification(R.drawable.playbackpause, getText(R.string.pause) + " " + song_title, song_artist );
		mp.pause();
		mp_status = PAUSED;
	}
	
	private void AP_resume() {
		ShowNotification(R.drawable.playbackstart, song_title, song_artist);
		mp.start();
		mp_status = PLAYING;
	}
	
	private void ShowNotification(int icon, String scrollText, String text) {
		Notification notification = new Notification(icon, scrollText, System.currentTimeMillis());
		Intent i = new Intent(this, AudioPlayer.class);
		//i = i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, i, 0);
		notification.setLatestEventInfo(this, text, scrollText, contentIntent);
		nm.notify(NOTIFY_ID, notification);
	}
	
	private final MCSInterface.Stub mBinder = new MCSInterface.Stub(){
		public void playAudioFile() {
			try {
				playFile();
			}
			catch (IndexOutOfBoundsException e) {
				Log.e(getString(R.string.app_name), e.getMessage());
			}
		}
		
		public void setCurrentPosition(int position) {
			curr_pos = position;
		}
		
		public int MPStatus() {
			return mp_status;
		}
		
		public void setDirPath(String path) {
			dirPath = path;
		}
		
		public void addAudioPlaylist(String song) {
			audio_plst.add(song);
		}
		
		public void delSong(int pos) {
			audio_plst.remove(pos);
		}
		
		public void clearAudioPlaylist() {
			audio_plst.clear();
		}
		
		public String getCurSongPath() {
			return dirPath + audio_plst.get(curr_pos);
		}
		
		public int getCurrentPosition() {
			return mp.getCurrentPosition();
		}
		
		public int getSongDuration() {
			return mp.getDuration();
		}
		
		public void setSongArtist(String song) {
			song_artist = song;
		}
		
		public void setSongAlbum(String album) {
			song_album = album;
		}
		
		public void setSongTitle(String title) {
			song_title = title;
		}
		
		public void pause() {
			AP_pause();
		}
		
		public void resume() {
			AP_resume();
		}
		
		public void stop() {
			nm.cancel(NOTIFY_ID);
			mp.stop();
			mp_status = STOPPED;
		}
		
		public void skipForward() {
			nextFile();
		}
		
		public void skipBack() {
			prevFile();
		}
		
		public void seekTo(int ms) {
			mp.seekTo(ms);
		}
	};
}
