package com.androidmc;

import java.util.ArrayList;

import android.app.Activity;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnBufferingUpdateListener;
import android.media.MediaPlayer.OnCompletionListener;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.SeekBar;
import android.widget.TextView;


public class VideoPlayer extends Activity  implements
OnBufferingUpdateListener, OnCompletionListener,
MediaPlayer.OnPreparedListener, SurfaceHolder.Callback, 
SeekBar.OnSeekBarChangeListener {

	private SurfaceView movie_screen;
	private SurfaceHolder holder;
	
	private ImageButton play, stop, prev, next;
	
	private TextView duration, full_duration;
	private int mProgressStatus = 0;
	private SeekBar seek;
	private Thread tProgress = null;
	private Handler mHandler = new Handler();
	int movDuration = 0;
	
	private MediaPlayer mp;
	private int mVideoWidth, mVideoHeight;
    private ArrayList<String> playlist;
    private int cur_pos;
    private String parentDir;
    
    private static final String TAG = "MediaPlayerDemo";
    
    private static final int STOPPED = 0;
    private static final int PLAYING = 1;
    private static final int PAUSED = 3;
    private int playerStatus = 0;
    
    
    @Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.vp_layout);
		
		Bundle extr = getIntent().getExtras();
		cur_pos = extr.getInt("POS", 0);
		playlist = extr.getStringArrayList("PLAYLIST");
		parentDir = extr.getString("DIR");
        
        play = (ImageButton)findViewById(R.id.btn_pp);
        stop = (ImageButton)findViewById(R.id.btn_stop);
        prev = (ImageButton)findViewById(R.id.btn_rew);
        next = (ImageButton)findViewById(R.id.btn_ff);
        play.setOnClickListener(mPlay);
        stop.setOnClickListener(mStop);
        prev.setOnClickListener(mPrev);
        next.setOnClickListener(mNext);
        
        duration = (TextView)findViewById(R.id.duration);
        full_duration = (TextView)findViewById(R.id.full_duration);
        
        seek = (SeekBar)findViewById(R.id.seek);
        seek.setOnSeekBarChangeListener(this);
        
        movie_screen = (SurfaceView)findViewById(R.id.surface);
        holder = movie_screen.getHolder();
        holder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
        holder.addCallback(this);
        mp = new MediaPlayer();
	}
	
    @Override
	protected void onStop() {
    	if(tProgress != null)
			tProgress.interrupt();
    	super.onStop();
		if (mp != null) {
            mp.release();
            mp = null;
        }
		
	}

    private int GetProgress() {
    	return mp.getCurrentPosition();
    }
    
    private void PlayVideo()
    {
    	try {
			mp.reset();
	        mp.setDataSource(parentDir + "/" + playlist.get(cur_pos));
	        mp.setDisplay(holder);
	        mp.prepare();
	        mp.setOnBufferingUpdateListener(this);
	        mp.setOnCompletionListener(this);
	        mp.setOnPreparedListener(this);
	        mp.setAudioStreamType(AudioManager.STREAM_MUSIC);
	        playerStatus = PLAYING;
	        play.setImageResource(R.drawable.pause);
    	}
    	catch(Exception e) 
    	{
    		Log.e(TAG, "error: " + e.getMessage(), e);
    	}
    }
    
    private void NextMov() {
    	if(++cur_pos >= playlist.size())
    		cur_pos = 0;
    	PlayVideo();
    }
    
    private void PrevMov() {
    	if (mp.getCurrentPosition() < 3000 && cur_pos >= 1) {
			--cur_pos;
			PlayVideo();
		} 
		else
			PlayVideo();
    }
    
    public void onBufferingUpdate(MediaPlayer arg0, int percent) {
        Log.d(TAG, "onBufferingUpdate percent:" + percent);

    }

    public void onCompletion(MediaPlayer arg0) {
        Log.d(TAG, "onCompletion called");
        NextMov();
    }

	@SuppressWarnings("deprecation")
	public void onPrepared(MediaPlayer mediaplayer) {
        Log.d(TAG, "onPrepared called");
        mVideoWidth = mp.getVideoWidth();
        mVideoHeight = mp.getVideoHeight();
        if (mVideoWidth != 0 && mVideoHeight != 0) {
            holder.setFixedSize(mVideoWidth, mVideoHeight);
            movDuration = mp.getDuration();
            seek.setMax(movDuration);
            full_duration.setText(" / " + MCUtils.MillisecToMinSec(movDuration));
            mp.start();
            if(tProgress != null) {
            	tProgress.resume();
            }
            else {
	            tProgress = new Thread(new Runnable() {
	            	public void run() {
		        		while (mProgressStatus < movDuration) {
		        			mProgressStatus = GetProgress();
		        			mHandler.post(new Runnable() {
		        				public void run() {
				        			seek.setProgress(mProgressStatus);
				        			duration.setText(MCUtils.MillisecToMinSec(mProgressStatus));
				        		}
		        			});
		        			try {
		        				Thread.sleep(1000);
		        			}
		        			catch(InterruptedException e) {
		        				return;
		        			}
		        		}
	            	}
	            });
	            tProgress.setPriority(Thread.MIN_PRIORITY);
	            tProgress.start();
            }
         }
    }

    public void surfaceChanged(SurfaceHolder surfaceholder, int i, int j, int k) {
    }

    @SuppressWarnings("deprecation")
	public void surfaceDestroyed(SurfaceHolder surfaceholder) {
    }


    public void surfaceCreated(SurfaceHolder holder) {
        PlayVideo();
    }
    
    private OnClickListener mPlay = new OnClickListener()
    {
    	@SuppressWarnings("deprecation")
		public void onClick(View v)
    	{
    		if(playerStatus == PLAYING) {
    			mp.pause();
    			tProgress.suspend();
    			playerStatus = PAUSED;
    			play.setImageResource(R.drawable.play);
    		}
    		else if (playerStatus == PAUSED) {
    			mp.start();
    			tProgress.resume();
    			playerStatus = PLAYING;
    			play.setImageResource(R.drawable.pause);
    		}
    		else if (playerStatus == STOPPED)
	    		PlayVideo();
    	}
    };
    
    private OnClickListener mStop = new OnClickListener()
    {
    	@SuppressWarnings("deprecation")
		public void onClick(View v)
    	{
    		mp.stop();
    		tProgress.suspend();
    		play.setImageResource(R.drawable.play);
    		playerStatus = STOPPED;
    	}
    };
    
    private OnClickListener mPrev = new OnClickListener()
    {
    	@SuppressWarnings("deprecation")
		public void onClick(View v)
    	{
    		tProgress.suspend();
    		PrevMov();
    	}
    };
    
    private OnClickListener mNext = new OnClickListener()
    {
    	@SuppressWarnings("deprecation")
		public void onClick(View v)
    	{
    		tProgress.suspend();
    		NextMov();
    	}
    };
    
    @SuppressWarnings("deprecation")
	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch) {
    	if(tProgress != null)
    		tProgress.resume();
    }

    @SuppressWarnings("deprecation")
	public void onStartTrackingTouch(SeekBar seekBar) {
    	if(tProgress != null)
    		tProgress.suspend();
    }

    @SuppressWarnings("deprecation")
	public void onStopTrackingTouch(SeekBar seekBar) {
    	mp.seekTo(seekBar.getProgress());
    }
}
